import os
import sys
import argparse
import json
from glob import glob
import mimetypes as mt

import itkdb

def set_hybrid_stage(client, sn, altID, new_stage):

    # Get Component 
        
    comp = client.get('getComponent', json={"component" : sn, "alternativeIdentifier" : altID})
    stage = comp["currentStage"]["code"]
    
    # Get component DB ID
    comp_name = comp["code"]
    
    
    # Get children
    has_children = False
    if comp["children"]:
        child_ids = [c["component"]["code"] for c in comp["children"] if c["component"]]
        children  = client.get('getComponentBulk', json={'component' : child_ids})
        has_children = True
    
    # Set new stage
    print(f"Setting stage {new_stage} for array {sn} ...")
    client.post("setComponentStage", json={'component' : comp_name , 'stage' : new_stage})
    if has_children:
        for c in children:
            cid = c["code"]
            cstage = c["currentStage"]["code"]
            #print(cid, cstage, stage)

            if cstage == stage:
                print(f"... setting stage for child {cid}")
                client.post("setComponentStage", json={'component' : cid, 'stage' : new_stage})
            else:
                print(f"... skipping child in different stage {cstage}")
    
    return


if __name__ == "__main__":

    # Parse command line
    parser = argparse.ArgumentParser(description="Script for uploading strip hyrbid test results")
    parser.add_argument("sn", help="SN of hybrid array or file contaiting multiple SNs, one per line")
    parser.add_argument("stage", help="New stage to move to")
    parser.add_argument("--alternativeID", default = False, help = "Alternative ID used (true/false)")
    parser.add_argument("--accessCode1", default = None, help = "Access Code 1 (can also be specified via ITKDB_ACCESS_CODE1 env var)")
    parser.add_argument("--accessCode2", default = None, help = "Access Code 2 (can also be specified via ITKDB_ACCESS_CODE2 env var)")
    parser.add_argument("--project", "-p", default = "S", help = "Project code (S/P)")
    args = parser.parse_args()


    # Authenticate
    if args.accessCode1 and args.accessCode2:
        u = itkdb.core.User(accessCode1=args.accessCode1, accessCode2=args.accessCode2)
        client = itkdb.Client(user = u)
    elif "ITKDB_ACCESS_CODE1" in os.environ and "ITKDB_ACCESS_CODE1" in os.environ:
        client = itkdb.Client()
    else:
        sys.exit("ERROR: Must provide access codes, either via arguments or environment variables")

    client.user.authenticate()
    user = client.get('getUser', json={'userIdentity': client.user.identity})
    print ('>>> Hello {} {}, welcome to the strip hybrid stage updater script'.format(user["firstName"], user["lastName"]))

    if os.path.isfile(args.sn):
        with open(args.sn) as f:
            for sn in f:
                set_hybrid_stage(client, sn.strip(), args.alternativeID, args.stage)
    else:
        set_hybrid_stage(client, args.sn, args.alternativeID, args.stage)

            
        
    
